/******************************************************************************
 * Copyright (c) 1996 Netscape Communications. All rights reserved.
 * Copyright (C) 1998 A. A. Babaylov <babolo@relcom.eu.net> (@BABOLO V.M 0.2iB)
 * With many thanks to Lawrence T. Hoff (hoff@bnl.gov) (UMP 1.01)
 ******************************************************************************/

#include <stdio.h>                       /* sprintf(), fprintf(), etc. */
#include <unistd.h>                      /* fork(), system(), unlink(), etc. */
#include <signal.h>                      /* SIGHUP */
#include <fcntl.h>                       /* O_RDWR */
#include <sys/types.h>                   /* pid_t */
#include <sys/wait.h>                    /* wait() */
#include <sys/time.h>                    /* timeval */
#include <sys/resource.h>                /* maximum # of file descriptors */

/*
** Stuff for the NPP_SetWindow method:
*/
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <NP/npapi.h>
extern int timiditymain(int, const char **);

typedef struct _PluginInstance
{   pid_t pid;                                           /* ns_timidity's ID */
} PluginInstance;

#define babolo_width 32
#define babolo_height 8
static char babolo_bits[] =
{0x3e,0x00,0x00,0x00,0x60,0x46,0x46,0x22,0xc6,0xaa,0xaa,0x52,0xdb,0xa6,0xa6,0x52
,0xdb,0xea,0xaa,0x52,0xdb,0xaa,0xaa,0x52,0xdb,0xa6,0x46,0x26,0x76,0x00,0x00,0x00
};

#define TRACE(N) /* {FILE *f=fopen("/tmp/trace","a");fprintf(f," %s\n",N);fclose(f);} */

static pid_t
startimidity(const char *filename)
{   pid_t pid; TRACE(filename);
    pid = fork(); if(pid != (pid_t)0) return pid;
    {        /* OK, now let's get ourselves somewhat removed from our parent */
        int fd, sig;
        struct rlimit rlp;

                    /* start by closing all file descriptors (except stderr) */
        (void) getrlimit(RLIMIT_NOFILE, &rlp);
        for(fd = 3; fd < rlp.rlim_cur; fd++) (void) close(fd);

        (void) freopen("/dev/null", "r", stdin);
        (void) freopen("/dev/null", "w", stdout);

                        /* then remove an netscape-installed signal handlers */
        for(sig=0; sig < 64; sig++)signal(sig, SIG_DFL);
    }
    {                                             /* Now play the sound file */
        char const *argv[2];
        argv[0] = "timidity";
        argv[1] = filename;
        (void)timiditymain(2, argv);
    }
    _exit(0);
}

/*****************************************************************************
 *                           Usual NP interface                              *
 *****************************************************************************/

char* NPP_GetMIMEDescription(void)
{   TRACE("NPP_GetMIMEDescription");
    return(
        "audio/midi:mid,midi:NS-timidity-0.2iB;"
        "audio/x-midi:mid,midi:"
        "<a href=http://zx-museum.org.ru/~babolo/>@BABOLO</a> 1998.02.08;"
    );
}

NPError NPP_GetValue(void *future, NPPVariable variable, void *value)
{   NPError err = NPERR_NO_ERROR;

    TRACE("NPP_GetValue");
    switch (variable)
    {case NPPVpluginNameString:
        *((char **)value) = "UNIX MIDI plugin"; break;
    case NPPVpluginDescriptionString:
        *((char **)value) =
            "Plays MIDI, using the timidity toolkit, via \"/dev/dsp\". "
            "You must install "
            "<a href=http://www.cgs.fi/~tt/timidity/>timidity</a> first. "
            "By default, timidity configuration files are in the directory "
            "\"/usr/local/bin/timidity\". "
        ;break;
    default: err = NPERR_GENERIC_ERROR;
    }
    return err;
}

NPError NPP_Initialize(void){ TRACE("NPP_Initialize"); return NPERR_NO_ERROR; }
jref NPP_GetJavaClass() { TRACE("NPP_GetJavaClass"); return NULL; }
void NPP_Shutdown(void) { TRACE("NPP_Shutdown"); }

NPError NPP_New( NPMIMEType pluginType, NPP instance, uint16 mode
               , int16 argc, char* argn[], char* argv[], NPSavedData* saved
               )
{   PluginInstance* This;

    TRACE("NPP_New");
    if (instance == NULL) return NPERR_INVALID_INSTANCE_ERROR;
    instance->pdata = NPN_MemAlloc(sizeof(PluginInstance));
    This = (PluginInstance*) instance->pdata;
    if (This == NULL) return NPERR_OUT_OF_MEMORY_ERROR;
    This->pid = -1;
    return NPERR_NO_ERROR;
}

NPError NPP_Destroy(NPP instance, NPSavedData** save)
{   PluginInstance* This;

    TRACE("NPP_Destroy");
    if (instance == NULL) return NPERR_INVALID_INSTANCE_ERROR;
    This = (PluginInstance*) instance->pdata;
    if ((This != NULL) && (This->pid != (pid_t)-1))
    {   kill(This->pid, SIGHUP);
        while(waitpid((pid_t)-1, NULL, WNOHANG)>0);
        NPN_MemFree(instance->pdata);
        instance->pdata = NULL;
    }
    return NPERR_NO_ERROR;
}

NPError NPP_NewStream( NPP instance, NPMIMEType type, NPStream *stream
                     , NPBool seekable, uint16 *stype
                     )
{   TRACE("NPP_NewStream"); *stype = NP_ASFILEONLY; return NPERR_NO_ERROR;
}

void NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{   PluginInstance* This;
    TRACE("NPP_StreamAsFile");
    if (instance != NULL)
    {   This = (PluginInstance*) instance->pdata;
        This->pid = startimidity(fname);
}   }

/*****************************************************************************
 *                        Nothing interesting below                          *
 *****************************************************************************/

NPError NPP_SetWindow(NPP instance, NPWindow* window)
{   TRACE("NPP_SetWindow"); return NPERR_NO_ERROR;
}   /* SDK- */

int32 STREAMBUFSIZE = 0X0FFFFFFF; /* If we are reading from a file in NPAsFile
				   * mode so we can take any size stream in our
				   * write call (since we ignore it) */

int32 NPP_WriteReady(NPP instance, NPStream *stream)
{   TRACE("NPP_WriteReady"); return STREAMBUFSIZE;
}   /* SDK- */

int32 NPP_Write( NPP instance, NPStream *stream
               , int32 offset, int32 len, void *buffer
               )
{   TRACE("NPP_Write"); return len;          /* The number of bytes accepted */
}   /* SDK- */

NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{   TRACE("NPP_DestroyStream"); return NPERR_NO_ERROR;
}   /* SDK- */

void NPP_Print(NPP instance, NPPrint* printInfo)
{   TRACE("NPP_Print");
    if  (   (printInfo != NULL)
        &&  (instance != NULL)
        &&  (printInfo->mode == NP_FULL)
        )
        printInfo->print.fullPrint.pluginPrinted = FALSE;
}   /* SDK- */

#include <NP/npunix.c>

